@@ -16,16 +16,28 @@
 #include <asterisk/lock.h>
 #include <asterisk/app.h>
 
+/*** DOCUMENTATION
+	<application name="SayCountPL" language="en_US">
+		<synopsis>
+			Say Polish counting words.
+		</synopsis>
+		<syntax>
+			<parameter name="word1" required="true" />
+			<parameter name="word2" required="true" />
+			<parameter name="word5" required="true" />
+			<parameter name="number" required="true" />
+		</syntax>
+		<description>
+			<para>Polish grammar has some funny rules for counting words. for example 1 zloty,
+			2 zlote, 5 zlotych. This application will take the words for 1, 2-4 and 5 and
+			decide based on grammar rules which one to use with the number you pass to it.</para>
+			<para>Example: SayCountPL(zloty,zlote,zlotych,122) will give: zlote</para>
+		</description>
+	</application>
+
+ ***/
 static char *app = "SayCountPL";
 
-static char *synopsis = "Say Polish counting words";
-
-static char *descrip =
-"Polish grammar has some funny rules for counting words. for example 1 zloty,\n"
-"2 zlote, 5 zlotych. This application will take the words for 1, 2-4 and 5 and\n"
-"decide based on grammar rules which one to use with the number you pass to it.\n\n"
-"Example: SayCountPL(zloty,zlote,zlotych,122) will give: zlote\n";
-
 static int saywords(struct ast_channel *chan, char *word1, char *word2, char *word5, int num)
 {
 	/* Put this in a separate proc because it's bound to change */
@@ -94,7 +106,7 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	return ast_register_application(app, sayword_exec, synopsis, descrip);
+	return ast_register_application_xml(app, sayword_exec);
 }
 
 AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Say polish counting words");
