@@ -39,6 +39,7 @@
 #include <asterisk/cli.h>
 #include <asterisk/strings.h>
 #include <asterisk/linkedlists.h>
+#include <asterisk/threadstorage.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -55,6 +56,11 @@
 #include <errno.h>
 
 #define DATE_FORMAT "%Y-%m-%d %T"
+
+AST_THREADSTORAGE(sql1_buf);
+AST_THREADSTORAGE(sql2_buf);
+AST_THREADSTORAGE(escape_buf);
+
 static char *desc = "MySQL CDR Backend";
 static char *name = "mysql";
 static char *config = "cdr_mysql.conf";
@@ -83,6 +89,7 @@ static AST_LIST_HEAD_STATIC(unload_strings, unload_string);
 struct column {
 	char *name;
 	char *cdrname;
+	char *staticvalue;
 	char *type;
 	AST_LIST_ENTRY(column) list;
 };
@@ -112,16 +119,16 @@ static char *handle_cli_cdr_mysql_status(struct ast_cli_entry *e, int cmd, struc
 		char status[256], status2[100] = "";
 		int ctime = time(NULL) - connect_time;
 		if (dbport)
-			snprintf(status, 255, "Connected to %s@%s, port %d", dbname->str, hostname->str, dbport);
+			snprintf(status, 255, "Connected to %s@%s, port %d", ast_str_buffer(dbname), ast_str_buffer(hostname), dbport);
 		else if (dbsock)
-			snprintf(status, 255, "Connected to %s on socket file %s", dbname->str, S_OR(dbsock->str, "default"));
+			snprintf(status, 255, "Connected to %s on socket file %s", ast_str_buffer(dbname), S_OR(ast_str_buffer(dbsock), "default"));
 		else
-			snprintf(status, 255, "Connected to %s@%s", dbname->str, hostname->str);
+			snprintf(status, 255, "Connected to %s@%s", ast_str_buffer(dbname), ast_str_buffer(hostname));
 
-		if (!ast_strlen_zero(dbuser->str))
-			snprintf(status2, 99, " with username %s", dbuser->str);
-		if (!ast_strlen_zero(dbtable->str))
-			snprintf(status2, 99, " using table %s", dbtable->str);
+		if (!ast_strlen_zero(ast_str_buffer(dbuser)))
+			snprintf(status2, 99, " with username %s", ast_str_buffer(dbuser));
+		if (ast_str_strlen(dbtable))
+			snprintf(status2, 99, " using table %s", ast_str_buffer(dbtable));
 		if (ctime > 31536000) {
 			ast_cli(a->fd, "%s%s for %d years, %d days, %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 31536000, (ctime % 31536000) / 86400, (ctime % 86400) / 3600, (ctime % 3600) / 60, ctime % 60);
 		} else if (ctime > 86400) {
@@ -150,8 +157,7 @@ static struct ast_cli_entry cdr_mysql_status_cli[] = {
 
 static int mysql_log(struct ast_cdr *cdr)
 {
-	char *sql1 = ast_calloc(1, 4096), *sql2 = ast_calloc(1, 2048);
-	int sql1size = 4096, sql2size = 2048;
+	struct ast_str *sql1 = ast_str_thread_get(&sql1_buf, 1024), *sql2 = ast_str_thread_get(&sql2_buf, 1024);
 	int retries = 5;
 #if MYSQL_VERSION_ID >= 50013
 	my_bool my_bool_true = 1;
@@ -179,19 +185,19 @@ db_reconnect:
 		}
 #endif
 		if (ssl_ca || ssl_cert || ssl_key) {
-			mysql_ssl_set(&mysql, ssl_key ? ssl_key->str : NULL, ssl_cert ? ssl_cert->str : NULL, ssl_ca ? ssl_ca->str : NULL, NULL, NULL);
+			mysql_ssl_set(&mysql, ssl_key ? ast_str_buffer(ssl_key) : NULL, ssl_cert ? ast_str_buffer(ssl_cert) : NULL, ssl_ca ? ast_str_buffer(ssl_ca) : NULL, NULL, NULL);
 		}
-		if (mysql_real_connect(&mysql, hostname->str, dbuser->str, password->str, dbname->str, dbport, dbsock && !ast_strlen_zero(dbsock->str) ? dbsock->str : NULL, ssl_ca ? CLIENT_SSL : 0)) {
+		if (mysql_real_connect(&mysql, ast_str_buffer(hostname), ast_str_buffer(dbuser), ast_str_buffer(password), ast_str_buffer(dbname), dbport, dbsock && ast_str_strlen(dbsock) ? ast_str_buffer(dbsock) : NULL, ssl_ca ? CLIENT_SSL : 0)) {
 			connected = 1;
 			connect_time = time(NULL);
 			records = 0;
 			if (dbcharset) {
-				snprintf(sql1, sizeof(sql1), "SET NAMES '%s'", dbcharset->str);
-				mysql_real_query(&mysql, sql1, strlen(sql1));
-				ast_debug(1, "SQL command as follows: %s\n", sql1);
+				ast_str_set(&sql1, 0, "SET NAMES '%s'", ast_str_buffer(dbcharset));
+				mysql_real_query(&mysql, ast_str_buffer(sql1), ast_str_strlen(sql1));
+				ast_debug(1, "SQL command as follows: %s\n", ast_str_buffer(sql1));
 			}
 		} else {
-			ast_log(LOG_ERROR, "Cannot connect to database server %s: (%d) %s\n", hostname->str, mysql_errno(&mysql), mysql_error(&mysql));
+			ast_log(LOG_ERROR, "Cannot connect to database server %s: (%d) %s\n", ast_str_buffer(hostname), mysql_errno(&mysql), mysql_error(&mysql));
 			connected = 0;
 		}
 	} else {
@@ -220,12 +226,12 @@ db_reconnect:
 	if (connected) {
 		int column_count = 0;
 		char *cdrname;
-		char workspace[2048], *value = NULL, *ptr;
-		int sql2len;
+		char workspace[2048], *value = NULL;
 		struct column *entry;
+		struct ast_str *escape = ast_str_thread_get(&escape_buf, 16);
 
-		snprintf(sql1, sql1size, "INSERT INTO %s (", dbtable ? dbtable->str : "cdr");
-		strcpy(sql2, ") VALUES ('");
+		ast_str_set(&sql1, 0, "INSERT INTO %s (", AS_OR(dbtable, "cdr"));
+		ast_str_set(&sql2, 0, ") VALUES (");
 
 		AST_RWLIST_RDLOCK(&columns);
 		AST_RWLIST_TRAVERSE(&columns, entry, list) {
@@ -252,16 +258,11 @@ db_reconnect:
 			}
 
 			/* Construct SQL */
-			if (strlen(sql1) + 2 + strlen(entry->name) > sql1size) {
-				char *tmp = ast_realloc(sql1, sql1size * 2);
-				if (!tmp)
-					goto log_exit;
-				sql1size *= 2;
-				sql1 = tmp;
-			}
 
 			/* Need the type and value to determine if we want the raw value or not */
-			if ((!strcmp(cdrname, "start") ||
+			if (entry->staticvalue) {
+				value = ast_strdupa(entry->staticvalue);
+			} else if ((!strcmp(cdrname, "start") ||
 				 !strcmp(cdrname, "answer") ||
 				 !strcmp(cdrname, "end") ||
 				 !strcmp(cdrname, "disposition") ||
@@ -272,57 +273,35 @@ db_reconnect:
 				 strstr(entry->type, "double") ||
 				 strstr(entry->type, "real") ||
 				 strstr(entry->type, "numeric") ||
-				 strstr(entry->type, "fixed")))
+				 strstr(entry->type, "fixed"))) {
 				ast_cdr_getvar(cdr, cdrname, &value, workspace, sizeof(workspace), 0, 1);
-			else
+			} else {
 				ast_cdr_getvar(cdr, cdrname, &value, workspace, sizeof(workspace), 0, 0);
-
-			if (strlen(sql2) + (value ? strlen(value) * 2 : 0) + 4 > sql2size) {
-				char *tmp = ast_realloc(sql2, sql2size * 2);
-				if (!tmp)
-					goto log_exit;
-				sql2size *= 2;
-				sql2 = tmp;
 			}
+
 			if (value) {
+				size_t valsz;
+
 				if (column_count++) {
-					strcat(sql1, ",");
-					strcat(sql2, "','");
+					ast_str_append(&sql1, 0, ",");
+					ast_str_append(&sql2, 0, ",");
 				}
 
-				strcat(sql1, entry->name);
+				ast_str_make_space(&escape, (valsz = strlen(value)) * 2 + 1);
+				mysql_real_escape_string(&mysql, ast_str_buffer(escape), value, valsz);
 
-				/*!\note We're manually escaping here, to ensure that we know exactly
-				 * how much space is used.  Since we only accept ASCII strings at this
-				 * point in time, there is no danger in this simplistic escape method,
-				 * but I wouldn't recommend this technique for other databases or if
-				 * we convert to an internal representation of UTF-8 sometime in the
-				 * future.
-				 */
-				sql2len = strlen(sql2);
-				for (ptr = value; *ptr; ptr++) {
-					if (*ptr == '\\' || *ptr == '\'')
-						sql2[sql2len++] = '\\';
-					sql2[sql2len++] = *ptr;
-				}
-				sql2[sql2len] = '\0';
+				ast_str_append(&sql1, 0, "%s", entry->name);
+				ast_str_append(&sql2, 0, "'%s'", ast_str_buffer(escape));
 			}
 		}
 		AST_RWLIST_UNLOCK(&columns);
 
 		ast_debug(1, "Inserting a CDR record.\n");
-		if (strlen(sql1) + 3 + strlen(sql2) > sql1size) {
-			char *tmp = ast_realloc(sql1, strlen(sql1) + 3 + strlen(sql2));
-			if (!tmp)
-				goto log_exit;
-			sql1 = tmp;
-		}
-		strcat(sql1, sql2);
-		strcat(sql1, "')");
+		ast_str_append(&sql1, 0, "%s)", ast_str_buffer(sql2));
 
-		ast_debug(1, "SQL command as follows: %s\n", sql1);
+		ast_debug(1, "SQL command as follows: %s\n", ast_str_buffer(sql1));
 
-		if (mysql_real_query(&mysql, sql1, strlen(sql1))) {
+		if (mysql_real_query(&mysql, ast_str_buffer(sql1), ast_str_strlen(sql1))) {
 			ast_log(LOG_ERROR, "Failed to insert into database: (%d) %s\n", mysql_errno(&mysql), mysql_error(&mysql));
 			mysql_close(&mysql);
 			connected = 0;
@@ -331,9 +310,6 @@ db_reconnect:
 			totalrecords++;
 		}
 	}
-log_exit:
-	ast_free(sql1);
-	ast_free(sql2);
 	ast_mutex_unlock(&mysql_lock);
 	return 0;
 }
@@ -387,15 +363,15 @@ static int my_load_config_string(struct ast_config *cfg, const char *category, c
 		return -1;
 	}
 
+	tmp = ast_variable_retrieve(cfg, category, variable);
+
+	ast_str_set(field, 0, "%s", tmp ? tmp : def);
+
 	us->str = *field;
 
 	AST_LIST_LOCK(&unload_strings);
 	AST_LIST_INSERT_HEAD(&unload_strings, us, entry);
 	AST_LIST_UNLOCK(&unload_strings);
-	
-	tmp = ast_variable_retrieve(cfg, category, variable);
-
-	ast_str_set(field, 0, "%s", tmp ? tmp : def);
 
 	return 0;
 }
@@ -467,7 +443,7 @@ static int my_load_module(int reload)
 	res |= my_load_config_number(cfg, "global", "port", &dbport, 0);
 	res |= my_load_config_number(cfg, "global", "timeout", &timeout, 0);
 	res |= my_load_config_string(cfg, "global", "compat", &compat, "no");
-	if (ast_true(compat->str)) {
+	if (ast_true(ast_str_buffer(compat))) {
 		calldate_compat = 1;
 	} else {
 		calldate_compat = 0;
@@ -489,18 +465,18 @@ static int my_load_module(int reload)
 		ast_free(entry);
 	}
 
-	ast_debug(1, "Got hostname of %s\n", hostname->str);
+	ast_debug(1, "Got hostname of %s\n", ast_str_buffer(hostname));
 	ast_debug(1, "Got port of %d\n", dbport);
 	ast_debug(1, "Got a timeout of %d\n", timeout);
 	if (dbsock)
-		ast_debug(1, "Got sock file of %s\n", dbsock->str);
-	ast_debug(1, "Got user of %s\n", dbuser->str);
-	ast_debug(1, "Got dbname of %s\n", dbname->str);
-	ast_debug(1, "Got password of %s\n", password->str);
+		ast_debug(1, "Got sock file of %s\n", ast_str_buffer(dbsock));
+	ast_debug(1, "Got user of %s\n", ast_str_buffer(dbuser));
+	ast_debug(1, "Got dbname of %s\n", ast_str_buffer(dbname));
+	ast_debug(1, "Got password of %s\n", ast_str_buffer(password));
 	ast_debug(1, "%sunning in calldate compatibility mode\n", calldate_compat ? "R" : "Not r");
 
 	if (dbcharset) {
-		ast_debug(1, "Got DB charste of %s\n", dbcharset->str);
+		ast_debug(1, "Got DB charset of %s\n", ast_str_buffer(dbcharset));
 	}
 
 	mysql_init(&mysql);
@@ -516,12 +492,16 @@ static int my_load_module(int reload)
 	}
 #endif
 
-	if ((ssl_ca && !ast_strlen_zero(ssl_ca->str)) || (ssl_cert && !ast_strlen_zero(ssl_cert->str)) || (ssl_key && !ast_strlen_zero(ssl_key->str))) {
-		mysql_ssl_set (&mysql, ssl_key->str, ssl_cert->str, ssl_ca->str, NULL, NULL);
+	if ((ssl_ca && ast_str_strlen(ssl_ca)) || (ssl_cert && ast_str_strlen(ssl_cert)) || (ssl_key && ast_str_strlen(ssl_key))) {
+		mysql_ssl_set(&mysql,
+			ssl_key ? ast_str_buffer(ssl_key) : NULL,
+			ssl_cert ? ast_str_buffer(ssl_cert) : NULL,
+			ssl_ca ? ast_str_buffer(ssl_ca) : NULL,
+			NULL, NULL);
 	}
-	temp = dbsock && !ast_strlen_zero(dbsock->str) ? dbsock->str : NULL;
-	if (!mysql_real_connect(&mysql, hostname->str, dbuser->str, password->str, dbname->str, dbport, temp, ssl_ca && !ast_strlen_zero(ssl_ca->str) ? CLIENT_SSL : 0)) {
-		ast_log(LOG_ERROR, "Failed to connect to mysql database %s on %s.\n", dbname->str, hostname->str);
+	temp = dbsock && ast_str_strlen(dbsock) ? ast_str_buffer(dbsock) : NULL;
+	if (!mysql_real_connect(&mysql, ast_str_buffer(hostname), ast_str_buffer(dbuser), ast_str_buffer(password), ast_str_buffer(dbname), dbport, temp, ssl_ca && ast_str_strlen(ssl_ca) ? CLIENT_SSL : 0)) {
+		ast_log(LOG_ERROR, "Failed to connect to mysql database %s on %s.\n", ast_str_buffer(dbname), ast_str_buffer(hostname));
 		connected = 0;
 		records = 0;
 	} else {
@@ -530,13 +510,13 @@ static int my_load_module(int reload)
 		records = 0;
 		connect_time = time(NULL);
 		if (dbcharset) {
-			snprintf(sqldesc, sizeof(sqldesc), "SET NAMES '%s'", dbcharset->str);
+			snprintf(sqldesc, sizeof(sqldesc), "SET NAMES '%s'", ast_str_buffer(dbcharset));
 			mysql_real_query(&mysql, sqldesc, strlen(sqldesc));
 			ast_debug(1, "SQL command as follows: %s\n", sqldesc);
 		}
 
 		/* Get table description */
-		snprintf(sqldesc, sizeof(sqldesc), "DESC %s", dbtable ? dbtable->str : "cdr");
+		snprintf(sqldesc, sizeof(sqldesc), "DESC %s", dbtable ? ast_str_buffer(dbtable) : "cdr");
 		if (mysql_query(&mysql, sqldesc)) {
 			ast_log(LOG_ERROR, "Unable to query table description!!  Logging disabled.\n");
 			mysql_close(&mysql);
@@ -557,47 +537,69 @@ static int my_load_module(int reload)
 
 		while ((row = mysql_fetch_row(result))) {
 			struct column *entry;
-			int foundalias = 0;
+			char *cdrvar = "", *staticvalue = "";
 
 			ast_debug(1, "Got a field '%s' of type '%s'\n", row[0], row[1]);
-			/* Check for an alias */
-			for (var = ast_variable_browse(cfg, "aliases"); var; var = var->next) {
-				if (strcasecmp(var->value, row[0])) {
-					continue;
+			/* Check for an alias or a static value */
+			for (var = ast_variable_browse(cfg, "columns"); var; var = var->next) {
+				if (strncmp(var->name, "alias", 5) == 0 && strcasecmp(var->value, row[0]) == 0 ) {
+					char *alias = ast_strdupa(var->name + 5);
+					cdrvar = ast_strip(alias);
+					ast_verb(3, "Found alias %s for column %s\n", cdrvar, row[0]);
+					break;
+				} else if (strncmp(var->name, "static", 6) == 0 && strcasecmp(var->value, row[0]) == 0) {
+					char *item = ast_strdupa(var->name + 6);
+					item = ast_strip(item);
+					if (item[0] == '"' && item[strlen(item) - 1] == '"') {
+						/* Remove surrounding quotes */
+						item[strlen(item) - 1] = '\0';
+						item++;
+					}
+					staticvalue = item;
 				}
+			}
 
-				if (!(entry = ast_calloc(1, sizeof(*entry) + strlen(var->name) + 1 + strlen(var->value) + 1 + strlen(row[1]) + 1))) {
-					continue;
-				}
-
-				entry->cdrname = (char *)entry + sizeof(*entry);
-				entry->name = (char *)entry + sizeof(*entry) + strlen(var->name) + 1;
-				entry->type = (char *)entry + sizeof(*entry) + strlen(var->name) + 1 + strlen(var->value) + 1;
-				strcpy(entry->cdrname, var->name);
-				strcpy(entry->name, var->value);
-				strcpy(entry->type, row[1]);
-
-				AST_LIST_INSERT_TAIL(&columns, entry, list);
-				ast_log(LOG_NOTICE, "Found an alias from CDR variable %s to DB column %s, type %s\n", entry->cdrname, entry->name, entry->type);
-				foundalias = 1;
+			entry = ast_calloc(sizeof(char), sizeof(*entry) + strlen(row[0]) + 1 + strlen(cdrvar) + 1 + strlen(staticvalue) + 1 + strlen(row[1]) + 1);
+			if (!entry) {
+				ast_log(LOG_ERROR, "Out of memory creating entry for column '%s'\n", row[0]);
+				res = -1;
 				break;
 			}
 
-			if (!foundalias && (entry = ast_calloc(1, sizeof(*entry) + strlen(row[0]) + 1 + strlen(row[1]) + 1))) {
-				entry->cdrname = (char *)entry + sizeof(*entry);
-				entry->name = (char *)entry + sizeof(*entry);
-				entry->type = (char *)entry + sizeof(*entry) + strlen(row[0]) + 1;
-				strcpy(entry->name, row[0]);
-				strcpy(entry->type, row[1]);
+			entry->name = (char *)entry + sizeof(*entry);
+			strcpy(entry->name, row[0]);
 
-				AST_LIST_INSERT_TAIL(&columns, entry, list);
-				ast_log(LOG_NOTICE, "Found a DB column %s, type %s\n", entry->name, entry->type);
+			if (!ast_strlen_zero(cdrvar)) {
+				entry->cdrname = entry->name + strlen(row[0]) + 1;
+				strcpy(entry->cdrname, cdrvar);
+			} else { /* Point to same place as the column name */
+				entry->cdrname = (char *)entry + sizeof(*entry);
 			}
+
+			if (!ast_strlen_zero(staticvalue)) {
+				entry->staticvalue = entry->cdrname + strlen(entry->cdrname) + 1;
+				strcpy(entry->staticvalue, staticvalue);
+				ast_debug(1, "staticvalue length: %d\n", (int) strlen(staticvalue) );
+				entry->type = entry->staticvalue + strlen(entry->staticvalue) + 1;
+			} else {
+				entry->type = entry->cdrname + strlen(entry->cdrname) + 1;
+			}
+			strcpy(entry->type, row[1]);
+
+			ast_debug(1, "Entry name '%s'\n", entry->name);
+			ast_debug(1, "   cdrname '%s'\n", entry->cdrname);
+			ast_debug(1, "    static '%s'\n", entry->staticvalue);
+			ast_debug(1, "      type '%s'\n", entry->type);
+
+			AST_LIST_INSERT_TAIL(&columns, entry, list);
 		}
 		mysql_free_result(result);
 	}
 	AST_RWLIST_UNLOCK(&columns);
 	ast_config_destroy(cfg);
+	if (res < 0) {
+		return AST_MODULE_LOAD_FAILURE;
+	}
 
 	res = ast_cdr_register(name, desc, mysql_log);
 	if (res) {
