@@ -68,6 +68,14 @@
 #define	READHANDLE	0
 #define	WRITEHANDLE	1
 
+#define ESCAPE_STRING(buf, var) \
+	do { \
+		if ((valsz = strlen(var)) * 2 + 1 > ast_str_size(buf)) { \
+			ast_str_make_space(&(buf), valsz * 2 + 1); \
+		} \
+		mysql_real_escape_string(&dbh->handle, ast_str_buffer(buf), var, valsz); \
+	} while (0)
+
 AST_THREADSTORAGE(sql_buf);
 AST_THREADSTORAGE(sql2_buf);
 AST_THREADSTORAGE(find_buf);
@@ -214,8 +222,8 @@ static struct tables *find_table(const char *database, const char *tablename)
 		return NULL;
 	}
 
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
-		ast_log(LOG_ERROR, "Failed to query database columns: %s\n", mysql_error(&dbh->handle));
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
+		ast_log(LOG_ERROR, "Failed to query database '%s', table '%s' columns: %s\n", database, tablename, mysql_error(&dbh->handle));
 		release_database(dbh);
 		AST_LIST_UNLOCK(&mysql_tables);
 		return NULL;
@@ -245,7 +253,7 @@ static struct tables *find_table(const char *database, const char *tablename)
 			}
 
 			if (!(column = ast_calloc(1, sizeof(*column) + strlen(fname) + strlen(ftype) + strlen(fdflt) + 3))) {
-				ast_log(LOG_ERROR, "Unable to allocate column element for %s, %s\n", tablename, fname);
+				ast_log(LOG_ERROR, "Unable to allocate column element %s for %s\n", fname, tablename);
 				destroy_table(table);
 				release_database(dbh);
 				AST_LIST_UNLOCK(&mysql_tables);
@@ -314,7 +322,7 @@ static struct ast_variable *realtime_mysql(const char *database, const char *tab
 	struct ast_variable *var=NULL, *prev=NULL;
 
 	if (!(dbh = find_database(database, 0))) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: %s\n", database);
+		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: %s (check res_mysql.conf)\n", database);
 		return NULL;
 	}
 
@@ -347,25 +355,23 @@ static struct ast_variable *realtime_mysql(const char *database, const char *tab
 	else 
 		op = "";
 
-	ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-	mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'", table, newparam, op, buf->str);
+	ESCAPE_STRING(buf, newval);
+	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'", table, newparam, op, ast_str_buffer(buf));
 	while ((newparam = va_arg(ap, const char *))) {
 		newval = va_arg(ap, const char *);
 		if (!strchr(newparam, ' ')) 
 			op = " ="; 
 		else
 			op = "";
-		ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-		mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-		ast_str_append(&sql, 0, " AND %s%s '%s'", newparam, op, buf->str);
+		ESCAPE_STRING(buf, newval);
+		ast_str_append(&sql, 0, " AND %s%s '%s'", newparam, op, ast_str_buffer(buf));
 	}
 	va_end(ap);
 
-	ast_debug(1, "MySQL RealTime: Retrieve SQL: %s\n", sql->str);
+	ast_debug(1, "MySQL RealTime: Retrieve SQL: %s\n", ast_str_buffer(sql));
 
 	/* Execution. */
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
 		ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database: %s\n", mysql_error(&dbh->handle));
 		release_database(dbh);
 		return NULL;
@@ -380,15 +386,15 @@ static struct ast_variable *realtime_mysql(const char *database, const char *tab
 				if (ast_strlen_zero(row[i]))
 					continue;
 				for (stringp = ast_strdupa(row[i]), chunk = strsep(&stringp, ";"); chunk; chunk = strsep(&stringp, ";")) {
-					if (chunk && !ast_strlen_zero(ast_strip(chunk))) {
-						if (prev) {
-							prev->next = ast_variable_new(fields[i].name, chunk, "");
-							if (prev->next) {
-								prev = prev->next;
-							}
-						} else {
-							prev = var = ast_variable_new(fields[i].name, chunk, "");
+					if (!chunk || ast_strlen_zero(ast_strip(chunk))) {
+						continue;
+					}
+					if (prev) {
+						if ((prev->next = ast_variable_new(fields[i].name, chunk, ""))) {
+							prev = prev->next;
 						}
+					} else {
+						prev = var = ast_variable_new(fields[i].name, chunk, "");
 					}
 				}
 			}
@@ -422,7 +428,7 @@ static struct ast_config *realtime_multi_mysql(const char *database, const char
 	struct ast_category *cat = NULL;
 
 	if (!(dbh = find_database(database, 0))) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s'\n", database);
+		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s' (check res_mysql.conf)\n", database);
 		return NULL;
 	}
 
@@ -469,15 +475,13 @@ static struct ast_config *realtime_multi_mysql(const char *database, const char
 	else
 		op = "";
 
-	ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-	mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'", table, newparam, op, buf->str);
+	ESCAPE_STRING(buf, newval);
+	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'", table, newparam, op, ast_str_buffer(buf));
 	while ((newparam = va_arg(ap, const char *))) {
 		newval = va_arg(ap, const char *);
 		if (!strchr(newparam, ' ')) op = " ="; else op = "";
-		ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-		mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-		ast_str_append(&sql, 0, " AND %s%s '%s'", newparam, op, buf->str);
+		ESCAPE_STRING(buf, newval);
+		ast_str_append(&sql, 0, " AND %s%s '%s'", newparam, op, ast_str_buffer(buf));
 	}
 
 	if (initfield) {
@@ -486,10 +490,10 @@ static struct ast_config *realtime_multi_mysql(const char *database, const char
 
 	va_end(ap);
 
-	ast_debug(1, "MySQL RealTime: Retrieve SQL: %s\n", sql->str);
+	ast_debug(1, "MySQL RealTime: Retrieve SQL: %s\n", ast_str_buffer(sql));
 
 	/* Execution. */
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
 		ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database: %s\n", mysql_error(&dbh->handle));
 		release_database(dbh);
 		ast_config_destroy(cfg);
@@ -543,7 +547,7 @@ static int update_mysql(const char *database, const char *tablename, const char
 	struct columns *column = NULL;
 
 	if (!(dbh = find_database(database, 1))) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s'.\n", database);
+		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s' (check res_mysql.conf)\n", database);
 		return -1;
 	}
 		
@@ -560,7 +564,7 @@ static int update_mysql(const char *database, const char *tablename, const char
 	}
 
 	if (!(column = find_column(table, keyfield))) {
-		ast_log(LOG_ERROR, "MySQL RealTime: Updating on column '%s', but that column does not exist within the table '%s'!\n", keyfield, tablename);
+		ast_log(LOG_ERROR, "MySQL RealTime: Updating on column '%s', but that column does not exist within the table '%s' (db '%s')!\n", keyfield, tablename, database);
 		release_table(table);
 		release_database(dbh);
 		return -1;
@@ -570,7 +574,7 @@ static int update_mysql(const char *database, const char *tablename, const char
 	newparam = va_arg(ap, const char *);
 	newval = va_arg(ap, const char *);
 	if (!newparam || !newval)  {
-		ast_log(LOG_WARNING, "MySQL RealTime: Realtime retrieval requires at least 1 parameter and 1 value to search on.\n");
+		ast_log(LOG_WARNING, "MySQL RealTime: Realtime update requires at least 1 parameter and 1 value to update.\n");
 		release_table(table);
 		release_database(dbh);
 		return -1;
@@ -578,7 +582,7 @@ static int update_mysql(const char *database, const char *tablename, const char
 
 	/* Check that the column exists in the table */
 	if (!(column = find_column(table, newparam))) {
-		ast_log(LOG_ERROR, "MySQL RealTime: Updating on column '%s', but that column does not exist within the table '%s'!\n", newparam, tablename);
+		ast_log(LOG_ERROR, "MySQL RealTime: Updating column '%s', but that column does not exist within the table '%s' (first pair MUST exist)!\n", newparam, tablename);
 		release_table(table);
 		release_database(dbh);
 		return -1;
@@ -594,9 +598,8 @@ static int update_mysql(const char *database, const char *tablename, const char
 	/* Create the first part of the query using the first parameter/value pairs we just extracted
 	   If there is only 1 set, then we have our query. Otherwise, loop thru the list and concat */
 
-	ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-	mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-	ast_str_set(&sql, 0, "UPDATE %s SET %s = '%s'", tablename, newparam, buf->str);
+	ESCAPE_STRING(buf, newval);
+	ast_str_set(&sql, 0, "UPDATE %s SET %s = '%s'", tablename, newparam, ast_str_buffer(buf));
 
 	/* If the column length isn't long enough, give a chance to lengthen it. */
 	if (strncmp(column->type, "char", 4) == 0 || strncmp(column->type, "varchar", 7) == 0) {
@@ -612,9 +615,8 @@ static int update_mysql(const char *database, const char *tablename, const char
 			continue;
 		}
 
-		ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-		mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-		ast_str_append(&sql, 0, ", %s = '%s'", newparam, buf->str);
+		ESCAPE_STRING(buf, newval);
+		ast_str_append(&sql, 0, ", %s = '%s'", newparam, ast_str_buffer(buf));
 
 		/* If the column length isn't long enough, give a chance to lengthen it. */
 		if (strncmp(column->type, "char", 4) == 0 || strncmp(column->type, "varchar", 7) == 0) {
@@ -623,15 +625,14 @@ static int update_mysql(const char *database, const char *tablename, const char
 	}
 	va_end(ap);
 
-	ast_str_make_space(&buf, (valsz = strlen(lookup)) * 2 + 1);
-	mysql_real_escape_string(&dbh->handle, buf->str, lookup, valsz);
-	ast_str_append(&sql, 0, " WHERE %s = '%s'", keyfield, buf->str);
+	ESCAPE_STRING(buf, lookup);
+	ast_str_append(&sql, 0, " WHERE %s = '%s'", keyfield, ast_str_buffer(buf));
 
-	ast_debug(1, "MySQL RealTime: Update SQL: %s\n", sql->str);
+	ast_debug(1, "MySQL RealTime: Update SQL: %s\n", ast_str_buffer(sql));
 
 	/* Execution. */
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database: %s\n", mysql_error(&dbh->handle));
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
+		ast_log(LOG_WARNING, "MySQL RealTime: Failed to update database: %s\n", mysql_error(&dbh->handle));
 		release_table(table);
 		release_database(dbh);
 		return -1;
@@ -652,6 +653,125 @@ static int update_mysql(const char *database, const char *tablename, const char
 	return (int)numrows;
 }
 
+static int update2_mysql(const char *database, const char *tablename, va_list ap)
+{
+	struct mysql_conn *dbh;
+	my_ulonglong numrows;
+	int first = 1;
+	const char *newparam, *newval;
+	size_t valsz;
+	struct ast_str *sql = ast_str_thread_get(&sql_buf, 100), *buf = ast_str_thread_get(&scratch_buf, 100);
+	struct ast_str *where = ast_str_thread_get(&sql2_buf, 100);
+	struct tables *table;
+	struct columns *column = NULL;
+
+	if (!tablename) {
+		ast_log(LOG_WARNING, "MySQL RealTime: No table specified.\n");
+		return -1;
+	}
+
+	if (!(dbh = find_database(database, 1))) {
+		ast_log(LOG_ERROR, "Invalid database specified: %s\n", database);
+		return -1;
+	}
+
+	if (!(table = find_table(database, tablename))) {
+		ast_log(LOG_ERROR, "Table '%s' does not exist!!\n", tablename);
+		release_database(dbh);
+		return -1;
+	}
+
+	if (!sql || !buf || !where) {
+		release_database(dbh);
+		release_table(table);
+		return -1;
+	}
+
+	ast_str_set(&sql, 0, "UPDATE %s SET", tablename);
+	ast_str_set(&where, 0, "WHERE");
+
+	/* Must connect to the server before anything else, as the escape function requires the mysql handle. */
+	if (!mysql_reconnect(dbh)) {
+		release_table(table);
+		release_database(dbh);
+		return -1;
+	}
+
+	while ((newparam = va_arg(ap, const char *))) {
+		if (!(column = find_column(table, newparam))) {
+			ast_log(LOG_ERROR, "Updating on column '%s', but that column does not exist within the table '%s'!\n", newparam, tablename);
+			release_table(table);
+			release_database(dbh);
+			return -1;
+		}
+		if (!(newval = va_arg(ap, const char *))) {
+			ast_log(LOG_ERROR, "Invalid arguments: no value specified for column '%s' on '%s@%s'\n", newparam, tablename, database);
+			release_table(table);
+			release_database(dbh);
+			return -1;
+		}
+		ESCAPE_STRING(buf, newval);
+		ast_str_append(&where, 0, "%s %s='%s'", first ? "" : " AND", newparam, ast_str_buffer(buf));
+		first = 0;
+
+		/* If the column length isn't long enough, give a chance to lengthen it. */
+		if (strncmp(column->type, "char", 4) == 0 || strncmp(column->type, "varchar", 7) == 0) {
+			internal_require(database, tablename, newparam, RQ_CHAR, valsz, SENTINEL);
+		}
+	}
+
+	first = 1;
+	while ((newparam = va_arg(ap, const char *))) {
+		if (!(newval = va_arg(ap, const char *))) {
+			ast_log(LOG_ERROR, "Invalid arguments: no value specified for column '%s' on '%s@%s'\n", newparam, tablename, database);
+			release_table(table);
+			release_database(dbh);
+			return -1;
+		}
+
+		/* If the column is not within the table, then skip it */
+		if (!(column = find_column(table, newparam))) {
+			ast_log(LOG_WARNING, "Attempted to update column '%s' in table '%s', but column does not exist!\n", newparam, tablename);
+			continue;
+		}
+
+		ESCAPE_STRING(buf, newval);
+		ast_str_append(&sql, 0, "%s %s = '%s'", first ? "" : ",", newparam, ast_str_buffer(buf));
+
+		/* If the column length isn't long enough, give a chance to lengthen it. */
+		if (strncmp(column->type, "char", 4) == 0 || strncmp(column->type, "varchar", 7) == 0) {
+			internal_require(database, tablename, newparam, RQ_CHAR, valsz, SENTINEL);
+		}
+	}
+	va_end(ap);
+	release_table(table);
+
+	ast_str_append(&sql, 0, " %s", ast_str_buffer(where));
+
+	ast_debug(1, "MySQL RealTime: Update SQL: %s\n", ast_str_buffer(sql));
+
+	/* Execution. */
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
+		ast_log(LOG_WARNING, "MySQL RealTime: Failed to update database: %s\n", mysql_error(&dbh->handle));
+		release_table(table);
+		release_database(dbh);
+		return -1;
+	}
+
+	numrows = mysql_affected_rows(&dbh->handle);
+	release_database(dbh);
+
+	ast_debug(1, "MySQL RealTime: Updated %llu rows on table: %s\n", numrows, tablename);
+
+	/* From http://dev.mysql.com/doc/mysql/en/mysql-affected-rows.html
+	 * An integer greater than zero indicates the number of rows affected
+	 * Zero indicates that no records were updated
+	 * -1 indicates that the query returned an error (although, if the query failed, it should have been caught above.)
+	*/
+
+	return (int)numrows;
+}
+ 
 static int store_mysql(const char *database, const char *table, va_list ap)
 {
 	struct mysql_conn *dbh;
@@ -663,7 +783,7 @@ static int store_mysql(const char *database, const char *table, va_list ap)
 	const char *newparam, *newval;
 
 	if (!(dbh = find_database(database, 1))) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s'.\n", database);
+		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s' (check res_mysql.conf)\n", database);
 		return -1;
 	}
 
@@ -687,34 +807,31 @@ static int store_mysql(const char *database, const char *table, va_list ap)
 	}
 	/* Create the first part of the query using the first parameter/value pairs we just extracted
 		If there is only 1 set, then we have our query. Otherwise, loop thru the list and concat */
-	ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-	mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-
+	ESCAPE_STRING(buf, newval);
 	ast_str_set(&sql, 0, "INSERT INTO %s (%s", table, newparam);
-	ast_str_set(&sql2, 0, ") VALUES ('%s'", buf->str);
+	ast_str_set(&sql2, 0, ") VALUES ('%s'", ast_str_buffer(buf));
 
 	internal_require(database, table, newparam, RQ_CHAR, valsz, SENTINEL);
 
 	while ((newparam = va_arg(ap, const char *))) {
 		if ((newval = va_arg(ap, const char *))) {
-			ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-			mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
+			ESCAPE_STRING(buf, newval);
 		} else {
 			valsz = 0;
 			ast_str_reset(buf);
 		}
 		if (internal_require(database, table, newparam, RQ_CHAR, valsz, SENTINEL) == 0) {
 			ast_str_append(&sql, 0, ", %s", newparam);
-			ast_str_append(&sql2, 0, ", '%s'", buf->str);
+			ast_str_append(&sql2, 0, ", '%s'", ast_str_buffer(buf));
 		}
 	}
 	va_end(ap);
-	ast_str_append(&sql, 0, "%s)", sql2->str);
-	ast_debug(1,"MySQL RealTime: Insert SQL: %s\n", sql->str);
+	ast_str_append(&sql, 0, "%s)", ast_str_buffer(sql2));
+	ast_debug(1,"MySQL RealTime: Insert SQL: %s\n", ast_str_buffer(sql));
 
 	/* Execution. */
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database: %s\n", mysql_error(&dbh->handle));
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
+		ast_log(LOG_WARNING, "MySQL RealTime: Failed to insert into database: %s\n", mysql_error(&dbh->handle));
 		release_database(dbh);
 		return -1;
 	}
@@ -743,7 +860,7 @@ static int destroy_mysql(const char *database, const char *table, const char *ke
 	const char *newparam, *newval;
 
 	if (!(dbh = find_database(database, 1))) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s'.\n", database);
+		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s' (check res_mysql.conf)\n", database);
 		return -1;
 	}
 
@@ -770,22 +887,20 @@ static int destroy_mysql(const char *database, const char *table, const char *ke
 
 	/* Create the first part of the query using the first parameter/value pairs we just extracted
 	   If there is only 1 set, then we have our query. Otherwise, loop thru the list and concat */
-	ast_str_make_space(&buf, (valsz = strlen(lookup)) * 2 + 1);
-	mysql_real_escape_string(&dbh->handle, buf->str, lookup, valsz);
-	ast_str_set(&sql, 0, "DELETE FROM %s WHERE %s = '%s'", table, keyfield, buf->str);
+	ESCAPE_STRING(buf, lookup);
+	ast_str_set(&sql, 0, "DELETE FROM %s WHERE %s = '%s'", table, keyfield, ast_str_buffer(buf));
 	while ((newparam = va_arg(ap, const char *))) {
 		newval = va_arg(ap, const char *);
-		ast_str_make_space(&buf, (valsz = strlen(newval)) * 2 + 1);
-		mysql_real_escape_string(&dbh->handle, buf->str, newval, valsz);
-		ast_str_append(&sql, 0, " AND %s = '%s'", newparam, buf->str);
+		ESCAPE_STRING(buf, newval);
+		ast_str_append(&sql, 0, " AND %s = '%s'", newparam, ast_str_buffer(buf));
 	}
 	va_end(ap);
 
-	ast_debug(1, "MySQL RealTime: Delete SQL: %s\n", sql->str);
+	ast_debug(1, "MySQL RealTime: Delete SQL: %s\n", ast_str_buffer(sql));
 
 	/* Execution. */
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database: %s\n", mysql_error(&dbh->handle));
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
+		ast_log(LOG_WARNING, "MySQL RealTime: Failed to delete from database: %s\n", mysql_error(&dbh->handle));
 		release_database(dbh);
 		return -1;
 	}
@@ -824,22 +939,22 @@ static struct ast_config *config_mysql(const char *database, const char *table,
 	}
 
 	if (!(dbh = find_database(database, 0))) {
-		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s'\n", database);
+		ast_log(LOG_WARNING, "MySQL RealTime: Invalid database specified: '%s' (check res_mysql.conf)\n", database);
 		return NULL;
 	}
 
-	ast_str_set(&sql, 0, "SELECT category, var_name, var_val, cat_metric FROM %s WHERE filename='%s' and commented=1 ORDER BY filename, cat_metric desc, var_metric asc, category, var_name, var_val, id", table, file);
+	ast_str_set(&sql, 0, "SELECT category, var_name, var_val, cat_metric FROM %s WHERE filename='%s' and commented=0 ORDER BY filename, cat_metric desc, var_metric asc, category, var_name, var_val, id", table, file);
 
-	ast_debug(1, "MySQL RealTime: Static SQL: %s\n", sql->str);
+	ast_debug(1, "MySQL RealTime: Static SQL: %s\n", ast_str_buffer(sql));
 
 	/* We now have our complete statement; Lets connect to the server and execute it. */
 	if (!mysql_reconnect(dbh)) {
 		return NULL;
 	}
 
-	if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
+	if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
 		ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database. Check debug for more info.\n");
-		ast_debug(1, "MySQL RealTime: Query: %s\n", sql->str);
+		ast_debug(1, "MySQL RealTime: Query: %s\n", ast_str_buffer(sql));
 		ast_debug(1, "MySQL RealTime: Query Failed because: %s\n", mysql_error(&dbh->handle));
 		release_database(dbh);
 		return NULL;
@@ -954,26 +1069,26 @@ static int modify_mysql(const char *database, const char *tablename, struct colu
 			res = -1;
 			break;
 		}
-		ast_str_set(&sql, 0, "ALTER TABLE %s MODIFY %s %s", tablename, column->name, typestr->str);
+		ast_str_set(&sql, 0, "ALTER TABLE %s MODIFY %s %s", tablename, column->name, ast_str_buffer(typestr));
 		if (!column->null) {
 			ast_str_append(&sql, 0, " NOT NULL");
 		}
 		if (!ast_strlen_zero(column->dflt)) {
-			ast_str_make_space(&escbuf, strlen(column->dflt) * 2 + 1);
-			mysql_real_escape_string(&dbh->handle, escbuf->str, column->dflt, strlen(column->dflt));
-			ast_str_append(&sql, 0, " DEFAULT '%s'", escbuf->str);
+			size_t valsz;
+			ESCAPE_STRING(escbuf, column->dflt);
+			ast_str_append(&sql, 0, " DEFAULT '%s'", ast_str_buffer(escbuf));
 		}
 
 		if (!mysql_reconnect(dbh)) {
-			ast_log(LOG_ERROR, "Unable to add column: %s\n", sql->str);
+			ast_log(LOG_ERROR, "Unable to add column: %s\n", ast_str_buffer(sql));
 			res = -1;
 			break;
 		}
 
 		/* Execution. */
-		if (mysql_real_query(&dbh->handle, sql->str, sql->used)) {
+		if (mysql_real_query(&dbh->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
 			ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database: %s\n", mysql_error(&dbh->handle));
-			ast_debug(1, "MySQL RealTime: Query: %s\n", sql->str);
+			ast_debug(1, "MySQL RealTime: Query: %s\n", ast_str_buffer(sql));
 			res = -1;
 		}
 	} while (0);
@@ -1211,19 +1326,19 @@ static int require_mysql(const char *database, const char *tablename, va_list ap
 				} else {
 					continue;
 				}
-				ast_str_set(&sql, 0, "ALTER TABLE %s ADD COLUMN %s %s", tablename, elm, fieldtype->str);
+				ast_str_set(&sql, 0, "ALTER TABLE %s ADD COLUMN %s %s", tablename, elm, ast_str_buffer(fieldtype));
 
 				ast_mutex_lock(&table->database->lock);
 				if (!mysql_reconnect(table->database)) {
 					ast_mutex_unlock(&table->database->lock);
-					ast_log(LOG_ERROR, "Unable to add column: %s\n", sql->str);
+					ast_log(LOG_ERROR, "Unable to add column: %s\n", ast_str_buffer(sql));
 					continue;
 				}
 
 				/* Execution. */
-				if (mysql_real_query(&table->database->handle, sql->str, sql->used)) {
+				if (mysql_real_query(&table->database->handle, ast_str_buffer(sql), ast_str_strlen(sql))) {
 					ast_log(LOG_WARNING, "MySQL RealTime: Failed to query database. Check debug for more info.\n");
-					ast_debug(1, "MySQL RealTime: Query: %s\n", sql->str);
+					ast_debug(1, "MySQL RealTime: Query: %s\n", ast_str_buffer(sql));
 					ast_debug(1, "MySQL RealTime: Query Failed because: %s\n", mysql_error(&table->database->handle));
 				} else {
 					table_altered = 1;
@@ -1249,6 +1364,7 @@ static struct ast_config_engine mysql_engine = {
 	.store_func = store_mysql,
 	.destroy_func = destroy_mysql,
 	.update_func = update_mysql,
+	.update2_func = update2_mysql,
 	.require_func = require_mysql,
 	.unload_func = unload_mysql,
 };
@@ -1313,10 +1429,12 @@ static int parse_config(int reload)
 	const char *catg;
 	struct mysql_conn *cur;
 
-	if (!(config = ast_config_load(RES_CONFIG_MYSQL_CONF, config_flags))) {
+	if ((config = ast_config_load(RES_CONFIG_MYSQL_CONF, config_flags)) == CONFIG_STATUS_FILEMISSING) {
 		return 0;
 	} else if (config == CONFIG_STATUS_FILEUNCHANGED) {
 		return 0;
+	} else if (config == CONFIG_STATUS_FILEINVALID) {
+		ast_log(LOG_ERROR, "Not %sloading " RES_CONFIG_MYSQL_CONF "\n", reload ? "re" : "");
 	}
 
 	AST_RWLIST_WRLOCK(&databases);
